@@ -180,8 +180,9 @@ windows_parse_config_entry(Repl_Agmt *ra, const char *type, Slapi_Entry *e)
 		{
 			windows_private_set_create_users(ra, PR_FALSE);
 		}
-		retval = 1;
 		slapi_ch_free((void**)&tmpstr);
+		prot_notify_agmt_changed(agmt_get_protocol(ra), (char *)agmt_get_long_name(ra));
+		retval = 1;
 	}
 	if (type == NULL || slapi_attr_types_equivalent(type,type_nsds7CreateNewGroups))
 	{
@@ -194,8 +195,9 @@ windows_parse_config_entry(Repl_Agmt *ra, const char *type, Slapi_Entry *e)
 		{
 			windows_private_set_create_groups(ra, PR_FALSE);
 		}
-		retval = 1;
 		slapi_ch_free((void**)&tmpstr);
+		prot_notify_agmt_changed(agmt_get_protocol(ra), (char *)agmt_get_long_name(ra));
+		retval = 1;
 	}
 	if (type == NULL || slapi_attr_types_equivalent(type,type_nsds7WindowsDomain))
 	{
@@ -214,6 +216,7 @@ windows_parse_config_entry(Repl_Agmt *ra, const char *type, Slapi_Entry *e)
 		if (NULL != tmpstr)
 		{
 			windows_private_set_sync_interval(ra,tmpstr);
+			prot_notify_agmt_changed(agmt_get_protocol(ra), (char *)agmt_get_long_name(ra));
 		}
 		slapi_ch_free_string(&tmpstr);
 		retval = 1;
@@ -240,6 +243,7 @@ windows_parse_config_entry(Repl_Agmt *ra, const char *type, Slapi_Entry *e)
 			windows_private_set_one_way(ra, ONE_WAY_SYNC_DISABLED);
 		}
 		slapi_ch_free((void**)&tmpstr);
+		prot_notify_agmt_changed(agmt_get_protocol(ra), (char *)agmt_get_long_name(ra));
 		retval = 1;
 	}
 	if (type == NULL || slapi_attr_types_equivalent(type,type_winsyncMoveAction))
@@ -266,8 +270,10 @@ windows_parse_config_entry(Repl_Agmt *ra, const char *type, Slapi_Entry *e)
 			windows_private_set_move_action(ra, MOVE_DOES_NOTHING);
 		}
 		slapi_ch_free((void**)&tmpstr);
+		prot_notify_agmt_changed(agmt_get_protocol(ra), (char *)agmt_get_long_name(ra));
 		retval = 1;
 	}
+
 	return retval;
 }
 
@@ -321,7 +327,12 @@ const char* windows_private_get_purl(const Repl_Agmt *ra)
 	char *hostname;
 
 	hostname = agmt_get_hostname(ra);
-	windows_purl = slapi_ch_smprintf("ldap://%s:%d", hostname, agmt_get_port(ra));
+	if(slapi_is_ipv6_addr(hostname)){
+		/* need to put brackets around the ipv6 address */
+		windows_purl = slapi_ch_smprintf("ldap://[%s]:%d", hostname, agmt_get_port(ra));
+	} else {
+		windows_purl = slapi_ch_smprintf("ldap://%s:%d", hostname, agmt_get_port(ra));
+	}
 	slapi_ch_free_string(&hostname);
 
 	return windows_purl;
