@@ -69,8 +69,8 @@ int ldap_quote_filter_value(
       int *outLen);
 
 
-static int search_one_berval(const char *baseDN, const char *attrName,
-		const struct berval *value, const char *requiredObjectClass, const char *target, void *txn);
+static int search_one_berval(Slapi_DN *baseDN, const char *attrName,
+		const struct berval *value, const char *requiredObjectClass, Slapi_DN *target);
 
 /*
  * ISSUES:
@@ -224,16 +224,16 @@ create_filter(const char *attribute, const struct berval *value, const char *req
  *   LDAP_OPERATIONS_ERROR - a server failure.
  */
 static int
-search(const char *baseDN, const char *attrName, Slapi_Attr *attr,
+search(Slapi_DN *baseDN, const char *attrName, Slapi_Attr *attr,
   struct berval **values, const char *requiredObjectClass,
-  const char *target, void *txn)
+  Slapi_DN *target)
 {
   int result;
 
 #ifdef DEBUG
     slapi_log_error(SLAPI_LOG_PLUGIN, plugin_name,
-      "SEARCH baseDN=%s attr=%s target=%s\n", baseDN, attrName, 
-      target?target:"None");
+                    "SEARCH baseDN=%s attr=%s target=%s\n", slapi_sdn_get_dn(baseDN), attrName, 
+                    target?slapi_sdn_get_dn(target):"None");
 #endif
 
   result = LDAP_SUCCESS;
@@ -260,7 +260,7 @@ search(const char *baseDN, const char *attrName, Slapi_Attr *attr,
 	{
 	  result = search_one_berval(baseDN, attrName,
 					slapi_value_get_berval(v),
-					requiredObjectClass, target, txn);
+					requiredObjectClass, target);
 	}
   }
   else
@@ -268,7 +268,7 @@ search(const char *baseDN, const char *attrName, Slapi_Attr *attr,
 	for (;*values != NULL && LDAP_SUCCESS == result; values++)
 	{
 	  result = search_one_berval(baseDN, attrName, *values, requiredObjectClass,
-					target, txn);
+					target);
 	}
   }
 
@@ -282,9 +282,9 @@ search(const char *baseDN, const char *attrName, Slapi_Attr *attr,
 
 
 static int
-search_one_berval(const char *baseDN, const char *attrName,
+search_one_berval(Slapi_DN *baseDN, const char *attrName,
 		const struct berval *value, const char *requiredObjectClass,
-		const char *target, void *txn)
+		Slapi_DN *target)
 {
 	int result;
     char *filter;
@@ -317,9 +317,8 @@ search_one_berval(const char *baseDN, const char *attrName,
       spb = slapi_pblock_new();
       if (!spb) { result = uid_op_error(2); break; }
 
-      slapi_search_internal_set_pb(spb, baseDN, LDAP_SCOPE_SUBTREE,
+      slapi_search_internal_set_pb_ext(spb, baseDN, LDAP_SCOPE_SUBTREE,
       	filter, attrs, 0 /* attrs only */, NULL, NULL, plugin_identity, 0 /* actions */);
-      slapi_pblock_set(spb, SLAPI_TXN, txn);
       slapi_search_internal_pb(spb);
 
       err = slapi_pblock_get(spb, SLAPI_PLUGIN_INTOP_RESULT, &sres);
@@ -341,18 +340,16 @@ search_one_berval(const char *baseDN, const char *attrName,
        */
       for(;*entries;entries++)
       {
-        char *ndn = slapi_entry_get_ndn(*entries); /* get the normalized dn */
-
 #ifdef DEBUG
         slapi_log_error(SLAPI_LOG_PLUGIN, plugin_name,
-          "SEARCH entry dn=%s\n", ndn);
+                        "SEARCH entry dn=%s\n", slapi_entry_get_dn(*entries));
 #endif
 
         /*
          * It is a Constraint Violation if any entry is found, unless
          * the entry is the target entry (if any).
          */
-        if (!target || strcmp(ndn, target) != 0)
+        if (!target || slapi_sdn_compare(slapi_entry_get_sdn(*entries), target) != 0)
         {
           result = LDAP_CONSTRAINT_VIOLATION;
           break;
@@ -395,7 +392,7 @@ search_one_berval(const char *baseDN, const char *attrName,
 static int
 searchAllSubtrees(int argc, char *argv[], const char *attrName,
   Slapi_Attr *attr, struct berval **values, const char *requiredObjectClass,
-  const char *dn, void *txn)
+  Slapi_DN *dn)
 {
   int result = LDAP_SUCCESS;
 
@@ -405,13 +402,17 @@ searchAllSubtrees(int argc, char *argv[], const char *attrName,
    */
   for(;argc > 0;argc--,argv++)
   {
+    Slapi_DN *sufdn = slapi_sdn_new_dn_byref(*argv);
     /*
      * The DN should already be normalized, so we don't have to
      * worry about that here.
      */
-    if (slapi_dn_issuffix(dn, *argv)) {
-      result = search(*argv, attrName, attr, values, requiredObjectClass, dn, txn);
+    if (slapi_sdn_issuffix(dn, sufdn)) {
+      result = search(sufdn, attrName, attr, values, requiredObjectClass, dn);
+      slapi_sdn_free(&sufdn);
       if (result) break;
+    } else {
+      slapi_sdn_free(&sufdn);
     }
   }
   return result;
@@ -498,27 +499,35 @@ getArguments(Slapi_PBlock *pb, char **attrName, char **markerObjectClass,
  *   LDAP_OPERATIONS_ERROR - a server failure.
  */
 static int
-findSubtreeAndSearch(char *parentDN, const char *attrName, Slapi_Attr *attr,
-  struct berval **values, const char *requiredObjectClass, const char *target,
-  const char *markerObjectClass, void *txn)
+findSubtreeAndSearch(Slapi_DN *parentDN, const char *attrName, Slapi_Attr *attr,
+  struct berval **values, const char *requiredObjectClass, Slapi_DN *target,
+  const char *markerObjectClass)
 {
   int result = LDAP_SUCCESS;
   Slapi_PBlock *spb = NULL;
+  Slapi_DN *curpar = slapi_sdn_new();
+  Slapi_DN *newpar = NULL;
 
-  while (NULL != (parentDN = slapi_dn_parent(parentDN)))
+  slapi_sdn_get_parent(parentDN, curpar);
+  while ((curpar != NULL) && (slapi_sdn_get_dn(curpar) != NULL))
   {
-        if ((spb = dnHasObjectClass(parentDN, markerObjectClass, txn, plugin_identity)))
+        if ((spb = dnHasObjectClass(curpar, markerObjectClass)))
         {
           freePblock(spb);
           /*
            * Do the search.   There is no entry that is allowed
            * to have the attribute already.
            */
-          result = search(parentDN, attrName, attr, values, requiredObjectClass,
-                          target, txn);
+          result = search(curpar, attrName, attr, values, requiredObjectClass,
+                          target);
           break;
         }
+        newpar = slapi_sdn_new();
+        slapi_sdn_copy(curpar, newpar);
+        slapi_sdn_get_parent(newpar, curpar);
+        slapi_sdn_free(&newpar);
   }
+  slapi_sdn_free(&curpar);
   return result;
 }
 
@@ -548,14 +557,12 @@ preop_add(Slapi_PBlock *pb)
     int err;
     char *markerObjectClass = NULL;
     char *requiredObjectClass = NULL;
-    const char *dn = NULL;
     Slapi_DN *sdn = NULL;
     int isupdatedn;
     Slapi_Entry *e;
     Slapi_Attr *attr;
     int argc;
     char **argv = NULL;
-    void *txn = NULL;
 
         /*
          * If this is a replication update, just be a noop.
@@ -567,7 +574,6 @@ preop_add(Slapi_PBlock *pb)
           break;
         }
 
-        slapi_pblock_get(pb, SLAPI_TXN, &txn);
     /*
      * Get the arguments
      */
@@ -596,10 +602,8 @@ preop_add(Slapi_PBlock *pb)
     err = slapi_pblock_get(pb, SLAPI_ADD_TARGET_SDN, &sdn);
     if (err) { result = uid_op_error(51); break; }
 
-    dn = slapi_sdn_get_dn(sdn);
-
 #ifdef DEBUG
-    slapi_log_error(SLAPI_LOG_PLUGIN, plugin_name, "ADD target=%s\n", dn);
+    slapi_log_error(SLAPI_LOG_PLUGIN, plugin_name, "ADD target=%s\n", slapi_sdn_get_dn(sdn));
 #endif
 
        /*
@@ -632,14 +636,14 @@ preop_add(Slapi_PBlock *pb)
         if (NULL != markerObjectClass)
         {
           /* Subtree defined by location of marker object class */
-                result = findSubtreeAndSearch((char *)dn, attrName, attr, NULL,
-                                              requiredObjectClass, dn,
-                                              markerObjectClass, txn);
+                result = findSubtreeAndSearch(sdn, attrName, attr, NULL,
+                                              requiredObjectClass, sdn,
+                                              markerObjectClass);
         } else
         {
           /* Subtrees listed on invocation line */
           result = searchAllSubtrees(argc, argv, attrName, attr, NULL,
-                                     requiredObjectClass, dn, txn);
+                                     requiredObjectClass, sdn);
         }
   END
 
@@ -706,12 +710,10 @@ preop_modify(Slapi_PBlock *pb)
     int modcount = 0;
     int ii;
     LDAPMod *mod;
-    const char *dn = NULL;
     Slapi_DN *sdn = NULL;
     int isupdatedn;
     int argc;
     char **argv = NULL;
-    void *txn = NULL;
 
     /*
      * If this is a replication update, just be a noop.
@@ -723,7 +725,6 @@ preop_modify(Slapi_PBlock *pb)
       break;
     }
 
-    slapi_pblock_get(pb, SLAPI_TXN, &txn);
     /*
      * Get the arguments
      */
@@ -780,12 +781,11 @@ preop_modify(Slapi_PBlock *pb)
     err = slapi_pblock_get(pb, SLAPI_MODIFY_TARGET_SDN, &sdn);
     if (err) { result = uid_op_error(11); break; }
 
-    dn = slapi_sdn_get_dn(sdn);
     /*
      * Check if it has the required object class
      */
     if (requiredObjectClass &&
-        !(spb = dnHasObjectClass(dn, requiredObjectClass, txn, plugin_identity))) {
+        !(spb = dnHasObjectClass(sdn, requiredObjectClass))) {
         break;
     }
 
@@ -803,14 +803,14 @@ preop_modify(Slapi_PBlock *pb)
         if (NULL != markerObjectClass)
         {
             /* Subtree defined by location of marker object class */
-            result = findSubtreeAndSearch((char *)dn, attrName, NULL, 
+            result = findSubtreeAndSearch(sdn, attrName, NULL, 
                                           mod->mod_bvalues, requiredObjectClass,
-                                          dn, markerObjectClass, txn);
+                                          sdn, markerObjectClass);
         } else
         {
             /* Subtrees listed on invocation line */
             result = searchAllSubtrees(argc, argv, attrName, NULL,
-                                       mod->mod_bvalues, requiredObjectClass, dn, txn);
+                                       mod->mod_bvalues, requiredObjectClass, sdn);
         }
     }
   END
@@ -862,7 +862,6 @@ preop_modrdn(Slapi_PBlock *pb)
     int err;
     char *markerObjectClass=NULL;
     char *requiredObjectClass=NULL;
-    const char *dn = NULL;
     Slapi_DN *sdn = NULL;
     Slapi_DN *superior;
     char *rdn;
@@ -871,7 +870,6 @@ preop_modrdn(Slapi_PBlock *pb)
     Slapi_Attr *attr;
     int argc;
     char **argv = NULL;
-    void *txn = NULL;
 
         /*
          * If this is a replication update, just be a noop.
@@ -883,7 +881,6 @@ preop_modrdn(Slapi_PBlock *pb)
           break;
         }
 
-        slapi_pblock_get(pb, SLAPI_TXN, &txn);
     /*
      * Get the arguments
      */
@@ -915,8 +912,6 @@ preop_modrdn(Slapi_PBlock *pb)
     err = slapi_pblock_get(pb, SLAPI_MODRDN_TARGET_SDN, &sdn);
     if (err) { result = uid_op_error(31); break; }
 
-    dn = slapi_sdn_get_dn(sdn);
-
     /* Get superior value - unimplemented in 3.0/4.0/5.0 DS */
     err = slapi_pblock_get(pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, &superior);
     if (err) { result = uid_op_error(32); break; }
@@ -942,7 +937,7 @@ preop_modrdn(Slapi_PBlock *pb)
 
     /* Get the entry that is being renamed so we can make a dummy copy
      * of what it will look like after the rename. */
-    err = slapi_search_internal_get_entry_ext(sdn, NULL, &e, plugin_identity, txn);
+    err = slapi_search_internal_get_entry(sdn, NULL, &e, plugin_identity);
     if (err != LDAP_SUCCESS) {
         result = uid_op_error(35);
         /* We want to return a no such object error if the target doesn't exist. */
@@ -953,7 +948,8 @@ preop_modrdn(Slapi_PBlock *pb)
     }
 
     /* Apply the rename operation to the dummy entry. */
-    err = slapi_entry_rename(e, rdn, deloldrdn, slapi_sdn_get_dn(superior));
+    /* slapi_entry_rename does not expect rdn normalized */
+    err = slapi_entry_rename(e, rdn, deloldrdn, superior);
     if (err != LDAP_SUCCESS) { result = uid_op_error(36); break; }
 
         /*
@@ -976,14 +972,14 @@ preop_modrdn(Slapi_PBlock *pb)
         if (NULL != markerObjectClass)
         {
           /* Subtree defined by location of marker object class */
-                result = findSubtreeAndSearch(slapi_entry_get_dn(e), attrName, attr, NULL,
-                                              requiredObjectClass, dn,
-                                              markerObjectClass, txn);
+                result = findSubtreeAndSearch(slapi_entry_get_sdn(e), attrName, attr, NULL,
+                                              requiredObjectClass, sdn,
+                                              markerObjectClass);
         } else
         {
           /* Subtrees listed on invocation line */
           result = searchAllSubtrees(argc, argv, attrName, attr, NULL,
-                                     requiredObjectClass, dn, txn);
+                                     requiredObjectClass, sdn);
         }
   END
   /* Clean-up */
@@ -1020,6 +1016,11 @@ int
 NSUniqueAttr_Init(Slapi_PBlock *pb)
 {
   int err = 0;
+  Slapi_Entry *plugin_entry = NULL;
+  char *plugin_type = NULL;
+  int preadd = SLAPI_PLUGIN_PRE_ADD_FN;
+  int premod = SLAPI_PLUGIN_PRE_MODIFY_FN;
+  int premdn = SLAPI_PLUGIN_PRE_MODRDN_FN;
 
   BEGIN
     int argc;
@@ -1038,6 +1039,16 @@ NSUniqueAttr_Init(Slapi_PBlock *pb)
     slapi_pblock_get (pb, SLAPI_PLUGIN_IDENTITY, &plugin_identity);
     /* PR_ASSERT (plugin_identity); */
 
+    if ((slapi_pblock_get(pb, SLAPI_PLUGIN_CONFIG_ENTRY, &plugin_entry) == 0) &&
+        plugin_entry &&
+        (plugin_type = slapi_entry_attr_get_charptr(plugin_entry, "nsslapd-plugintype")) &&
+        plugin_type && strstr(plugin_type, "betxn")) {
+        preadd = SLAPI_PLUGIN_BE_TXN_PRE_ADD_FN;
+        premod = SLAPI_PLUGIN_BE_TXN_PRE_MODIFY_FN;
+        premdn = SLAPI_PLUGIN_BE_TXN_PRE_MODRDN_FN;
+    }
+    slapi_ch_free_string(&plugin_type);
+
     /*
      * Get and normalize arguments
      */
@@ -1063,16 +1074,13 @@ NSUniqueAttr_Init(Slapi_PBlock *pb)
     if (err) break;
 
     /* Register functions */
-    err = slapi_pblock_set(pb, SLAPI_PLUGIN_PRE_ADD_FN,
-            (void*)preop_add);
+    err = slapi_pblock_set(pb, preadd, (void*)preop_add);
     if (err) break;
 
-    err = slapi_pblock_set(pb, SLAPI_PLUGIN_PRE_MODIFY_FN,
-            (void*)preop_modify);
+    err = slapi_pblock_set(pb, premod, (void*)preop_modify);
     if (err) break;
 
-    err = slapi_pblock_set(pb, SLAPI_PLUGIN_PRE_MODRDN_FN,
-            (void*)preop_modrdn);
+    err = slapi_pblock_set(pb, premdn, (void*)preop_modrdn);
     if (err) break;
 
   END
