@@ -651,6 +651,40 @@ dse_updateNumSubOfParent(struct dse *pdse, const Slapi_DN *child, int op)
 	slapi_sdn_done(&parent);
 }
 
+/* check if a file is valid, or if a provided backup file can be used.
+ * there is no way to determine if the file contents is usable, the only
+ * checks that can be done is that the file exists and that it is not size 0
+ */
+int
+dse_check_file(char *filename, char *backupname)
+{
+    int rc= 0; /* Fail */
+    PRFileInfo prfinfo;
+
+    if (PR_GetFileInfo( filename, &prfinfo ) == PR_SUCCESS) {
+	if ( prfinfo.size > 0)
+		return (1);
+	else {
+		rc = PR_Delete (filename);
+	}
+    }
+
+    if (backupname) 
+	rc = PR_Rename (backupname, filename);
+    else 
+	return (0);
+
+    if ( PR_GetFileInfo( filename, &prfinfo ) == PR_SUCCESS && prfinfo.size > 0 ) {
+	slapi_log_error(SLAPI_LOG_FATAL, "dse",
+             "The configuration file %s was restored from backup %s\n", filename, backupname);
+	return (1);
+    } else {
+        slapi_log_error(SLAPI_LOG_FATAL, "dse",
+              "The configuration file %s was not restored from backup %s, error %d\n",
+                                    filename, backupname, rc);
+	return (0);
+   }
+}
 static int
 dse_read_one_file(struct dse *pdse, const char *filename, Slapi_PBlock *pb,
         int primary_file )
@@ -669,27 +703,11 @@ dse_read_one_file(struct dse *pdse, const char *filename, Slapi_PBlock *pb,
 
     if ( (NULL != pdse) && (NULL != filename) )
     {
-        if ( (rc = PR_GetFileInfo( filename, &prfinfo )) != PR_SUCCESS )
-        {
-            /* the "real" file does not exist; see if there is a tmpfile */
-            if ( pdse->dse_tmpfile &&
-                 PR_GetFileInfo( pdse->dse_tmpfile, &prfinfo ) == PR_SUCCESS ) {
-                rc = PR_Rename(pdse->dse_tmpfile, filename);
-                if (rc == PR_SUCCESS) {
-                    slapi_log_error(SLAPI_LOG_FATAL, "dse",
-                                    "The configuration file %s was restored from backup %s\n",
-                                    filename, pdse->dse_tmpfile);
-                    rc = 1;
-                } else {
-                    slapi_log_error(SLAPI_LOG_FATAL, "dse",
-                                    "The configuration file %s was not restored from backup %s, error %d\n",
-                                    filename, pdse->dse_tmpfile, rc);
-                    rc = 0;
-                }
-            } else {
-                rc = 0; /* fail */
-            }
-        }
+        /* check if the "real" file exists and cam be used, if not try tmp as backup */
+	rc = dse_check_file((char *)filename, pdse->dse_tmpfile);
+	if (!rc) 
+	    rc = dse_check_file((char *)filename, pdse->dse_fileback);
+
         if ( (rc = PR_GetFileInfo( filename, &prfinfo )) != PR_SUCCESS )
         {
             slapi_log_error(SLAPI_LOG_FATAL, "dse",
@@ -1967,18 +1985,19 @@ dse_modify(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
         /* next, give the betxn plugins a crack at it */
         slapi_pblock_set(pb, SLAPI_RESULT_CODE, &returncode);
         slapi_pblock_set(pb, SLAPI_MODIFY_EXISTING_ENTRY, ecc);
-        plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN);
-        if (!returncode) {
-            slapi_pblock_get(pb, SLAPI_RESULT_CODE, &returncode);
-        }
-        if (returncode && !returntext[0]) {
-            char *ldap_result_message = NULL;
-            slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
-            if (ldap_result_message && ldap_result_message[0]) {
-                PL_strncpyz(returntext, ldap_result_message, sizeof(returntext));
-            }
-        }
         if (need_be_postop) {
+            plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN);
+            if (!returncode) {
+                slapi_pblock_get(pb, SLAPI_RESULT_CODE, &returncode);
+            }
+            if (returncode && !returntext[0]) {
+                char *ldap_result_message = NULL;
+                slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
+                if (ldap_result_message && ldap_result_message[0]) {
+                    PL_strncpyz(returntext, ldap_result_message, sizeof(returntext));
+                }
+            }
+
             plugin_call_plugins(pb, SLAPI_PLUGIN_BE_POST_MODIFY_FN);
             if (!returncode) {
                 slapi_pblock_get(pb, SLAPI_RESULT_CODE, &returncode);
@@ -2198,10 +2217,11 @@ dse_add(Slapi_PBlock *pb) /* JCM There should only be one exit point from this f
             returncode = rc;
         }
     }
-    /* next, give the be txn plugins a crack at it */
-    slapi_pblock_set(pb, SLAPI_RESULT_CODE, &returncode);
-    plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_ADD_FN);
     if (need_be_postop) {
+        /* next, give the be txn plugins a crack at it */
+        slapi_pblock_set(pb, SLAPI_RESULT_CODE, &returncode);
+        plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_ADD_FN);
+
         /* finally, give the be plugins a crack at it */
         plugin_call_plugins(pb, SLAPI_PLUGIN_BE_POST_ADD_FN);
         if (!returncode) {
@@ -2287,7 +2307,6 @@ dse_delete(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
     }
 
     if(dse_call_callback(pdse, pb, SLAPI_OPERATION_DELETE, DSE_FLAG_PREOP, ec, NULL, &returncode,returntext)==SLAPI_DSE_CALLBACK_OK) {
-        slapi_pblock_get(pb, SLAPI_DELETE_BEPREOP_ENTRY, &orig_entry);
         slapi_pblock_set(pb, SLAPI_DELETE_BEPREOP_ENTRY, ec);
         slapi_pblock_set(pb, SLAPI_RESULT_CODE, &returncode);
         plugin_call_plugins(pb, SLAPI_PLUGIN_BE_PRE_DELETE_FN);
@@ -2306,13 +2325,15 @@ dse_delete(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
                 }
             }
         }
-        slapi_pblock_set(pb, SLAPI_DELETE_BEPREOP_ENTRY, orig_entry);
+        /* Setting SLAPI_ENTRY_PRE_OP here,
+         * since some betxn postop may need the pre op entry. */
+        slapi_pblock_set(pb, SLAPI_ENTRY_PRE_OP, slapi_entry_dup(ec));
     } else {
         goto done;
     }
 
     dse_call_callback(pdse, pb, SLAPI_OPERATION_DELETE, DSE_FLAG_POSTOP, ec, NULL, &returncode, returntext);
- done:
+done:
     slapi_pblock_get(pb, SLAPI_DELETE_BEPOSTOP_ENTRY, &orig_entry);
     slapi_pblock_set(pb, SLAPI_DELETE_BEPOSTOP_ENTRY, ec);
     /* make sure OPRETURN and RESULT_CODE are set */
@@ -2325,11 +2346,12 @@ dse_delete(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
             returncode = rc;
         }
     }
-    plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN);
-    if (!returncode) {
-        slapi_pblock_get(pb, SLAPI_RESULT_CODE, &returncode);
-    }
     if (need_be_postop) {
+        plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN);
+        if (!returncode) {
+            slapi_pblock_get(pb, SLAPI_RESULT_CODE, &returncode);
+        }
+
         /* finally, give the be plugins a crack at it */
         plugin_call_plugins(pb, SLAPI_PLUGIN_BE_POST_DELETE_FN);
         if (!returncode) {
@@ -2345,9 +2367,6 @@ dse_delete(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
     }
     slapi_pblock_set(pb, SLAPI_DELETE_BEPOSTOP_ENTRY, orig_entry);
     slapi_send_ldap_result( pb, returncode, NULL, returntext, 0, NULL );
-    if (ec) {
-        slapi_pblock_set( pb, SLAPI_ENTRY_PRE_OP, slapi_entry_dup( ec ));
-    }
     return dse_delete_return(returncode, ec);
 }
 
