@@ -165,55 +165,59 @@ convert_to_openldap_uri(const char *hostname_or_uri, int port, const char *proto
     char *my_copy = NULL;
     char *start = NULL;
     char *iter = NULL;
+    char *ptr = NULL;
     char *s = NULL;
     const char *brkstr = " ";
+    int done = 0;
 
     if (!hostname_or_uri) {
-	return NULL;
+	    return NULL;
+    }
+
+    if(slapi_is_ipv6_addr(hostname_or_uri)){
+        /* We need to encapsulate the ipv6 addr with brackets  */
+        my_copy = slapi_ch_smprintf("[%s]",hostname_or_uri);
+    } else {
+        my_copy = slapi_ch_strdup(hostname_or_uri);
     }
 
-    my_copy = slapi_ch_strdup(hostname_or_uri);
     /* see if hostname_or_uri is an ldap uri */
     if (!proto && !PL_strncasecmp(my_copy, "ldap", 4)) {
-	start = my_copy + 4;
-	if ((*start == 's') || (*start == 'i')) {
-	    start++;
-	}
-	if (!PL_strncmp(start, "://", 3)) {
-	    *start = '\0';
-	    proto = my_copy;
-	    start += 3;
-	} else {
-	    slapi_log_error(SLAPI_LOG_FATAL, "convert_to_openldap_uri",
-			    "The given LDAP URI [%s] is not valid\n", hostname_or_uri);
-	    goto end;
-	}
+        start = my_copy + 4;
+        if ((*start == 's') || (*start == 'i')) {
+            start++;
+        }
+        if (!PL_strncmp(start, "://", 3)) {
+            *start = '\0';
+            proto = my_copy;
+            start += 3;
+        } else {
+            slapi_log_error(SLAPI_LOG_FATAL, "convert_to_openldap_uri",
+                "The given LDAP URI [%s] is not valid\n", hostname_or_uri);
+            goto end;
+        }
     } else if (!proto) {
-	slapi_log_error(SLAPI_LOG_FATAL, "convert_to_openldap_uri",
-			"The given LDAP URI [%s] is not valid\n", hostname_or_uri);
-	goto end;
+	    slapi_log_error(SLAPI_LOG_FATAL, "convert_to_openldap_uri",
+            "The given LDAP URI [%s] is not valid\n", hostname_or_uri);
+        goto end;
     } else {
-	start = my_copy; /* just assume it's not a uri */
+        start = my_copy; /* just assume it's not a uri */
     }
 	    
-    for (s = ldap_utf8strtok_r(my_copy, brkstr, &iter); s != NULL;
-	 s = ldap_utf8strtok_r(NULL, brkstr, &iter)) {
-	char *ptr;
-	int last = 0;
-	/* strtok will grab the '/' at the end of the uri, if any,
-	   so terminate parsing there */
-	if ((ptr = strchr(s, '/'))) {
-	    *ptr = '\0';
-	    last = 1;
-	}
-	if (retstr) {
-	    retstr = PR_sprintf_append(retstr, "/ %s://%s", proto, s);
-	} else {
-	    retstr = PR_smprintf("%s://%s", proto, s);
-	}
-	if (last) {
-	    break;
-	}
+    for (s = ldap_utf8strtok_r(my_copy, brkstr, &iter); s != NULL; s = ldap_utf8strtok_r(NULL, brkstr, &iter)) {
+        /* strtok will grab the '/' at the end of the uri, if any,  so terminate parsing there */
+        if ((ptr = strchr(s, '/'))) {
+            *ptr = '\0';
+            done = 1;
+        }
+        if (retstr) {
+            retstr = PR_sprintf_append(retstr, "/ %s://%s", proto, s);
+        } else {
+            retstr = PR_smprintf("%s://%s", proto, s);
+        }
+        if (done) {
+            break;
+        }
     }
 
     /* add the port on the last one */
@@ -300,53 +304,56 @@ slapi_ldap_url_parse(const char *url, LDAPURLDesc **ludpp, int require_dn, int *
        replace all but the last colon with %3A
        Go to the 3rd '/' or to the end of the string (convert only the host:port part) */
     if (url) {
-	char *p = strstr(url, "://");
-	if (p) {
-	    int foundspace = 0;
-	    int coloncount = 0;
-	    char *lastcolon = NULL;
-	    p += 3;
-	    for (; *p && (*p != '/'); p++) {
-		if (*p == ' ') {
-		    foundspace = 1;
-		}
-		if (*p == ':') {
-		    coloncount++;
-		    lastcolon = p;
-		}
-	    }
-	    if (foundspace) {
-		char *src = NULL, *dest = NULL;
-		/* have to convert url */
-		/* len * 3 is way too much, but acceptable */
-		urlescaped = slapi_ch_calloc(strlen(url) * 3, sizeof(char));
-		dest = urlescaped;
-		/* copy the scheme */
-	        src = strstr(url, "://");
-		src += 3;
-		memcpy(dest, url, src-url);
-		dest += (src-url);
-		/* we have to convert all spaces to %20 - we have to convert
-		   all colons except the last one to %3A */
-		for (; *src; ++src) {
-		    if (src < p) {
-			if (*src == ' ') {
-			    memcpy(dest, "%20", 3);
-			    dest += 3;
-			} else if ((coloncount > 1) && (*src == ':') && (src != lastcolon)) {
-			    memcpy(dest, "%3A", 3);
-			    dest += 3;
-			} else {
-			    *dest++ = *src;
-			}
-		    } else {
-			*dest++ = *src;
-		    }
-		}
-		*dest = '\0';
-		url_to_use = urlescaped;
-	    }
-	}
+        char *p = strstr(url, "://");
+        if (p) {
+            int foundspace = 0;
+            int coloncount = 0;
+            char *lastcolon = NULL;
+
+            p += 3;
+            for (; *p && (*p != '/'); p++) {
+                if (*p == ' ') {
+                    foundspace = 1;
+                }
+                if (*p == ':') {
+                    coloncount++;
+                    lastcolon = p;
+                }
+            }
+            if (foundspace) {
+                char *src = NULL, *dest = NULL;
+
+                /* have to convert url, len * 3 is way too much, but acceptable */
+                urlescaped = slapi_ch_calloc(strlen(url) * 3, sizeof(char));
+                dest = urlescaped;
+                /* copy the scheme */
+                src = strstr(url, "://");
+                src += 3;
+                memcpy(dest, url, src-url);
+                dest += (src-url);
+                /*
+                 * we have to convert all spaces to %20 - we have to convert
+                 * all colons except the last one to %3A
+                 */
+                for (; *src; ++src) {
+                    if (src < p) {
+                        if (*src == ' ') {
+                            memcpy(dest, "%20", 3);
+                            dest += 3;
+                        } else if ((coloncount > 1) && (*src == ':') && (src != lastcolon)) {
+                            memcpy(dest, "%3A", 3);
+                            dest += 3;
+                        } else {
+                            *dest++ = *src;
+                        }
+                    } else {
+                        *dest++ = *src;
+                    }
+                }
+                *dest = '\0';
+                url_to_use = urlescaped;
+            }
+        }
     }
 #endif
 
@@ -384,30 +391,30 @@ slapi_ldap_url_parse(const char *url, LDAPURLDesc **ludpp, int require_dn, int *
 
 #if defined(USE_OPENLDAP)
     if (urlescaped && (*ludpp) && (*ludpp)->lud_host) {
-	/* have to unescape lud_host - can unescape in place */
-	char *p = strstr((*ludpp)->lud_host, "://");
-	if (p) {
-	    char *dest = NULL;
-	    p += 3;
-	    dest = p;
-	    /* up to the first '/', unescape the host */
-	    for (; *p && (*p != '/'); p++) {
-		if (!strncmp(p, "%20", 3)) {
-		    *dest++ = ' ';
-		    p += 2;
-		} else if (!strncmp(p, "%3A", 3)) {
-		    *dest++ = ':';
-		    p += 2;
-		} else {
-		    *dest++ = *p;
-		}
-	    }
-	    /* just copy the remainder of the host, if any */
-	    while (*p) {
-		*dest++ = *p++;
-	    }
-	    *dest = '\0';
-	}
+        /* have to unescape lud_host - can unescape in place */
+        char *p = strstr((*ludpp)->lud_host, "://");
+        if (p) {
+            char *dest = NULL;
+            p += 3;
+            dest = p;
+            /* up to the first '/', unescape the host */
+            for (; *p && (*p != '/'); p++) {
+                if (!strncmp(p, "%20", 3)) {
+                    *dest++ = ' ';
+                    p += 2;
+                } else if (!strncmp(p, "%3A", 3)) {
+                    *dest++ = ':';
+                    p += 2;
+                } else {
+                    *dest++ = *p;
+                }
+            }
+            /* just copy the remainder of the host, if any */
+            while (*p) {
+                *dest++ = *p++;
+            }
+            *dest = '\0';
+        }
     }
     slapi_ch_free_string(&urlescaped);
 #endif
@@ -1076,15 +1083,42 @@ slapi_ldap_bind(
 			mech ? mech : "SIMPLE",
 			bindid, creds);
 	if ((rc = ldap_sasl_bind(ld, bindid, mech, &bvcreds, serverctrls,
-				 NULL /* clientctrls */, &mymsgid))) {
+	                         NULL /* clientctrls */, &mymsgid))) {
+	    char *myhostname = NULL;
+	    char *copy = NULL;
+	    char *ptr = NULL;
+	    int myerrno = errno;
+	    int gaierr;
+
+	    ldap_get_option(ld, LDAP_OPT_HOST_NAME, &myhostname);
+	    if (myhostname) {
+	        ptr = strchr(myhostname, ':');
+	        if (ptr) {
+	            copy = slapi_ch_strdup(myhostname);
+	            *(copy + (ptr - myhostname)) = '\0';
+	            myhostname = copy;
+	        }
+	    }
+
+	    if (0 == myerrno) {
+	        struct addrinfo *result = NULL;
+	        gaierr = getaddrinfo(myhostname, NULL, NULL, &result);
+	        myerrno = errno;
+	        if (result) {
+	            freeaddrinfo(result);
+	        }
+	    }
 	    slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
 			    "Error: could not send bind request for id "
-			    "[%s] mech [%s]: error %d (%s) %d (%s) %d (%s)\n",
+			    "[%s] mech [%s]: error %d (%s) %d (%s) %d (%s \"%s\")\n",
 			    bindid ? bindid : "(anon)",
 			    mech ? mech : "SIMPLE",
 			    rc, ldap_err2string(rc),
 			    PR_GetError(), slapd_pr_strerror(PR_GetError()),
-			    errno, slapd_system_strerror(errno));
+			    myerrno ? myerrno : gaierr,
+			    myerrno ? slapd_system_strerror(myerrno) : gai_strerror(gaierr),
+			    myhostname ? myhostname : "unknown host");
+	    slapi_ch_free_string(&copy);
 	    goto done;
 	}
 
@@ -2270,3 +2304,15 @@ mozldap_ldap_explode_rdn( const char *rdn, const int notypes )
 	return( mozldap_ldap_explode( rdn, notypes, LDAP_RDN ) );
 }
 
+int
+slapi_is_ipv6_addr( const char *hostname ){
+    PRNetAddr addr;
+
+    if(PR_StringToNetAddr(hostname, &addr) == PR_SUCCESS &&
+       !PR_IsNetAddrType(&addr, PR_IpAddrV4Mapped) &&
+       addr.raw.family == PR_AF_INET6)
+    {
+        return 1;
+    }
+    return 0;
+}
