@@ -61,6 +61,9 @@
 #if defined( _WIN32 )
 #undef LDAPDebug
 #endif	/*  _WIN32 */
+#if defined( USE_MOZLDAP )
+#define LDAP_MOD_OP (0x0007)
+#endif /* USE_MOZLDAP */
 
 #include "slap.h"
 
@@ -197,7 +200,6 @@ char* slapi_encode_ext (Slapi_PBlock *pb, const Slapi_DN *sdn, char *value, char
 	{
 		pwpolicy = new_passwdPolicy(pb, (char*)slapi_sdn_get_ndn(sdn) );
 		pws_enc = pwpolicy->pw_storagescheme->pws_enc;
-		delete_passwdPolicy(&pwpolicy);
 
 		if (pws_enc == NULL)
 		{
@@ -354,8 +356,6 @@ pw_encodevals_ext( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals )
 		if (pwpolicy->pw_storagescheme) {
 			pws_enc = pwpolicy->pw_storagescheme->pws_enc;
 		}
-
-		delete_passwdPolicy(&pwpolicy);
 	}
 
 	/* Password scheme encryption function was not found */
@@ -675,7 +675,6 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw) {
 			  slapi_ch_free((void**)&prev_exp_date_str);
 			  pw_apply_mods(sdn, &smods);
 			  slapi_mods_done(&smods);
-			  delete_passwdPolicy(&pwpolicy);
 			  return 0;
 			}
 			
@@ -692,12 +691,9 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw) {
 	} else {
 		pw_apply_mods(sdn, &smods);
 		slapi_mods_done(&smods);
-		delete_passwdPolicy(&pwpolicy);
 		return 0;
 	}
 
-	delete_passwdPolicy(&pwpolicy);
-
 	timestr = format_genTime ( pw_exp_date );
 	slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "passwordExpirationTime", timestr);
 	slapi_ch_free((void **)&timestr);
@@ -732,7 +728,6 @@ check_pw_minage ( Slapi_PBlock *pb, const Slapi_DN *sdn, struct berval **vals)
 		/* retrieve the entry */
 		e = get_entry ( pb, dn );
 		if ( e == NULL ) {
-			delete_passwdPolicy(&pwpolicy);
 			return ( -1 );
 		}
 		/* get passwordAllowChangeTime attribute */
@@ -760,14 +755,12 @@ check_pw_minage ( Slapi_PBlock *pb, const Slapi_DN *sdn, struct berval **vals)
                         "within password minimum age", 0, NULL );
 				slapi_entry_free( e );
 				slapi_ch_free((void **) &cur_time_str );
-				delete_passwdPolicy(&pwpolicy);
 				return ( 1 );
 			}
         	slapi_ch_free((void **) &cur_time_str );
 		}
         slapi_entry_free( e );
 	}
-	delete_passwdPolicy(&pwpolicy);
 	return ( 0 );
 }
 
@@ -844,12 +837,10 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 								LDAP_PWPOLICY_INVALIDPWDSYNTAX );
 					}
 					pw_send_ldap_result ( pb, LDAP_CONSTRAINT_VIOLATION, NULL, errormsg, 0, NULL );
-					delete_passwdPolicy(&pwpolicy);
 					return( 1 );
 				} else {
 					/* We want to skip syntax checking since this is a pre-hashed
 					 * password from replication or the root DN. */
-					delete_passwdPolicy(&pwpolicy);
 					return( 0 );
 				}
 			}
@@ -866,7 +857,6 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 							LDAP_PWPOLICY_PWDTOOSHORT );
 				}
 				pw_send_ldap_result ( pb, LDAP_CONSTRAINT_VIOLATION, NULL, errormsg, 0, NULL );
-				delete_passwdPolicy(&pwpolicy);
 				return ( 1 );
 			}
 
@@ -981,7 +971,6 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 					    LDAP_PWPOLICY_INVALIDPWDSYNTAX );
 				}
 				pw_send_ldap_result ( pb, LDAP_CONSTRAINT_VIOLATION, NULL, errormsg, 0, NULL );
-				delete_passwdPolicy(&pwpolicy);
 				return ( 1 );
 			}
 		}
@@ -992,7 +981,6 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 		/* retrieve the entry */
 		e = get_entry ( pb, dn );
 		if ( e == NULL ) {
-			delete_passwdPolicy(&pwpolicy);
 			return ( -1 );
 		}
 
@@ -1012,7 +1000,6 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 						LDAP_CONSTRAINT_VIOLATION, NULL,
 						"password in history", 0, NULL );
 					slapi_entry_free( e ); 
-					delete_passwdPolicy(&pwpolicy);
 					return ( 1 );
 				}
 			}
@@ -1030,7 +1017,6 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 										   LDAP_CONSTRAINT_VIOLATION ,NULL,
 										   "password in history", 0, NULL);
 						slapi_entry_free( e ); 
-						delete_passwdPolicy(&pwpolicy);
 						return ( 1 );
 					}
 				} else 
@@ -1041,7 +1027,6 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 										   LDAP_CONSTRAINT_VIOLATION ,NULL,
 										   "password in history", 0, NULL);
 						slapi_entry_free( e ); 
-						delete_passwdPolicy(&pwpolicy);
 						return ( 1 );
 					}
 				}
@@ -1070,13 +1055,10 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 				slapi_entry_free( e );
 			}
 
-			delete_passwdPolicy(&pwpolicy);
 			return 1;
 		}
 	}
 
-	delete_passwdPolicy(&pwpolicy);
-
 	if ( mod_op ) {
 		/* free e only when called by modify operation */
 		slapi_entry_free( e ); 
@@ -1107,7 +1089,6 @@ update_pw_history( Slapi_PBlock *pb, const Slapi_DN *sdn, char *old_pw )
 	/* retrieve the entry */
 	e = get_entry ( pb, dn );
 	if ( e == NULL ) {
-		delete_passwdPolicy(&pwpolicy);
 		return ( 1 );
 	}
 
@@ -1173,7 +1154,6 @@ update_pw_history( Slapi_PBlock *pb, const Slapi_DN *sdn, char *old_pw )
 	slapi_ch_free((void **) &str );
 	slapi_ch_free((void **) &history_str );
 	slapi_entry_free( e );
-	delete_passwdPolicy(&pwpolicy);
 	return 0;
 }
 
@@ -1412,8 +1392,6 @@ add_password_attrs( Slapi_PBlock *pb, Operation *op, Slapi_Entry *e )
 		slapi_entry_attr_merge( e, "passwordallowchangetime", bvals );
 		slapi_ch_free((void **) &bv.bv_val );
 	}
-	
-	delete_passwdPolicy(&pwpolicy);
 }
 
 static int
@@ -1548,6 +1526,11 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 	slapdFrontendConfig_t *slapdFrontendConfig;
 	int optype = -1;
 
+	/* If we already allocated a pw policy, return it */
+	if(pb && pb->pwdpolicy){
+		return pb->pwdpolicy;
+	}
+
 	slapdFrontendConfig = getFrontendConfig();
 	pwdpolicy = (passwdPolicy *)slapi_ch_calloc(1, sizeof(passwdPolicy));
 
@@ -1625,10 +1608,13 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 				              "--local policy entry not found\n", dn);
 				goto done;
 			}
-        
+
 			/* set the default passwordLegacyPolicy setting */
 			pwdpolicy->pw_is_legacy = 1;
 
+			/* set passwordTrackUpdateTime */
+			pwdpolicy->pw_track_update_time = slapdFrontendConfig->pw_policy.pw_track_update_time;
+
 			for (slapi_entry_first_attr(pw_entry, &attr); attr;
 					slapi_entry_next_attr(pw_entry, attr, &attr))
 			{
@@ -1832,6 +1818,9 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 			if (pw_entry) {
 				slapi_entry_free(pw_entry);
 			}
+			if(pb){
+				pb->pwdpolicy = pwdpolicy;
+			}
 			return pwdpolicy;
 		} else if ( e ) { 
 			slapi_entry_free( e );
@@ -1839,15 +1828,18 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 	}
 
 done:
-	/* If we are here, that means we need to load the passwdPolicy
+	/*
+	 * If we are here, that means we need to load the passwdPolicy
 	 * structure from slapdFrontendconfig
 	 */
-
 	*pwdpolicy = slapdFrontendConfig->pw_policy;
 	pwdscheme = (struct pw_scheme *)slapi_ch_calloc(1, sizeof(struct pw_scheme));
 	*pwdscheme = *slapdFrontendConfig->pw_storagescheme;
 	pwdscheme->pws_name = strdup( slapdFrontendConfig->pw_storagescheme->pws_name );
 	pwdpolicy->pw_storagescheme = pwdscheme;
+	if(pb){
+		pb->pwdpolicy = pwdpolicy;
+	}
 
 	return pwdpolicy;
 
@@ -2188,14 +2180,9 @@ slapi_check_account_lock ( Slapi_PBlock *pb, Slapi_Entry * bind_target_entry, in
 
 notlocked:
 	/* account is not locked. */
-        if(check_password_policy)
-		delete_passwdPolicy(&pwpolicy);
-	return ( 0 );	
+	return (0);
 locked:
-	if(check_password_policy)
-		delete_passwdPolicy(&pwpolicy);
 	return (1);
-
 }
 
 /* The idea here is that these functions could allow us to have password
@@ -2350,3 +2337,241 @@ slapi_pwpolicy_is_reset(Slapi_PWPolicy *pwpolicy, Slapi_Entry *e)
 
     return is_reset;
 }
+
+/*
+ * Entry extension for unhashed password
+ */
+static int pw_entry_objtype = -1;
+static int pw_entry_handle = -1;
+
+struct slapi_pw_entry_ext {
+	Slapi_RWLock *pw_entry_lock;   /* necessary? */
+	Slapi_Value **pw_entry_values; /* stashed values */
+};
+
+/*
+ * constructor for the entry object extension.
+ */
+static void *
+pw_entry_constructor(void *object, void *parent)
+{
+	struct slapi_pw_entry_ext *pw_extp = NULL;
+	Slapi_RWLock *rwlock;
+	if ((rwlock = slapi_new_rwlock()) == NULL) {
+		slapi_log_error(SLAPI_LOG_FATAL, NULL,
+		                "pw_entry_constructor: slapi_new_rwlock() failed\n");
+		slapi_log_error(SLAPI_LOG_FATAL, NULL,
+		                "WARNING: the server cannot handle unhashed password.\n");
+		return NULL;
+	}
+	pw_extp = (struct slapi_pw_entry_ext *)slapi_ch_calloc(1,
+                                             sizeof(struct slapi_pw_entry_ext));
+	pw_extp->pw_entry_lock = rwlock;
+	return pw_extp;
+}
+
+/*
+ * destructor for the entry object extension.
+ */
+static void
+pw_entry_destructor(void *extension, void *object, void *parent)
+{
+	struct slapi_pw_entry_ext *pw_extp = (struct slapi_pw_entry_ext *)extension;
+
+	if (NULL == pw_extp) {
+		return;
+	}
+	
+	valuearray_free(&pw_extp->pw_entry_values);
+
+	if (pw_extp->pw_entry_lock) {
+		slapi_destroy_rwlock(pw_extp->pw_entry_lock);
+	}
+	slapi_ch_free((void **)&pw_extp);
+}
+
+/* Called once from main */
+void
+pw_exp_init ( void )
+{
+	if (slapi_register_object_extension(SLAPI_EXTMOD_PWPOLICY,
+	                                    SLAPI_EXT_ENTRY,
+	                                    pw_entry_constructor,
+	                                    pw_entry_destructor,
+	                                    &pw_entry_objtype,
+	                                    &pw_entry_handle) != 0) {
+		slapi_log_error(SLAPI_LOG_FATAL, NULL,
+		                "pw_init: slapi_register_object_extension failed; "
+		                "unhashed password is not able to access\n");
+	}
+}
+
+/* 
+ * The output value vals is not a copy.  
+ * Caller must duplicate it to use it for other than referring.
+ */
+int
+slapi_pw_get_entry_ext(Slapi_Entry *entry, Slapi_Value ***vals)
+{
+	struct slapi_pw_entry_ext *extp = NULL;
+
+	if (NULL == vals) {
+		slapi_log_error(SLAPI_LOG_FATAL, NULL,
+		                "slapi_pw_get_entry_ext: output param vals is NULL.\n");
+		return LDAP_PARAM_ERROR;
+	}
+	*vals = NULL;
+
+	if ((-1 == pw_entry_objtype) || (-1 == pw_entry_handle)) {
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "slapi_pw_get_entry_ext: pw_entry_extension is not "
+		                "registered\n");
+		return LDAP_OPERATIONS_ERROR;
+	}
+
+	extp = (struct slapi_pw_entry_ext *)slapi_get_object_extension(
+	                                                           pw_entry_objtype,
+	                                                           entry,
+	                                                           pw_entry_handle);
+	if ((NULL == extp) || (NULL == extp->pw_entry_values)) {
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "slapi_pw_get_entry_ext: "
+		                "pw_entry_extension is not set\n");
+		return LDAP_NO_SUCH_ATTRIBUTE;
+	}
+
+	slapi_rwlock_rdlock(extp->pw_entry_lock);
+	*vals = extp->pw_entry_values;
+	slapi_rwlock_unlock(extp->pw_entry_lock);
+	return LDAP_SUCCESS;
+}
+
+/* If vals is NULL, the stored extension is freed.  */
+/* If slapi_pw_set_entry_ext is successful, vals are consumed. */
+int
+slapi_pw_set_entry_ext(Slapi_Entry *entry, Slapi_Value **vals, int flags)
+{
+	struct slapi_pw_entry_ext *extp = NULL;
+
+	if ((-1 == pw_entry_objtype) || (-1 == pw_entry_handle)) {
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "slapi_pw_set_entry_ext: "
+		                "pw_entry_extension is not registered\n");
+		return LDAP_OPERATIONS_ERROR;
+	}
+
+	extp = (struct slapi_pw_entry_ext *)slapi_get_object_extension(
+	                                                           pw_entry_objtype,
+	                                                           entry,
+	                                                           pw_entry_handle);
+	if (NULL == extp) {
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "slapi_pw_set_entry_ext: "
+		                "pw_entry_extension is not set\n");
+		return LDAP_NO_SUCH_ATTRIBUTE;
+	}
+
+	slapi_rwlock_wrlock(extp->pw_entry_lock);
+	if (NULL == vals) { /* Set NULL; used for delete. */
+		valuearray_free(&extp->pw_entry_values); /* Null is taken care */
+	} else {
+		if (SLAPI_EXT_SET_REPLACE == flags) {
+			valuearray_free(&extp->pw_entry_values); /* Null is taken care */
+		}
+		/* Each (Slapi_Value *) in vals is passed in. */
+		valuearray_add_valuearray(&extp->pw_entry_values, vals,
+		                          SLAPI_VALUE_FLAG_PASSIN);
+		/* To keep the word "consumed", free vals part, as well. */
+		slapi_ch_free((void **)&vals);
+	}
+	slapi_rwlock_unlock(extp->pw_entry_lock);
+	return LDAP_SUCCESS;
+}
+
+int
+pw_copy_entry_ext(Slapi_Entry *src_e, Slapi_Entry *dest_e)
+{
+	struct slapi_pw_entry_ext *src_extp = NULL;
+	struct slapi_pw_entry_ext *dest_extp = NULL;
+
+	if ((-1 == pw_entry_objtype) || (-1 == pw_entry_handle)) {
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "pw_copy_entry_ext: "
+		                "pw_entry_extension is not registered\n");
+		return LDAP_OPERATIONS_ERROR;
+	}
+
+	src_extp = (struct slapi_pw_entry_ext *)slapi_get_object_extension(
+	                                                           pw_entry_objtype,
+	                                                           src_e,
+	                                                           pw_entry_handle);
+	if (NULL == src_extp) {
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "pw_copy_entry_ext: source pw_entry_extension is "
+		                "not set\n");
+		return LDAP_NO_SUCH_ATTRIBUTE;
+	}
+
+	slapi_rwlock_rdlock(src_extp->pw_entry_lock);
+	dest_extp = (struct slapi_pw_entry_ext *)slapi_get_object_extension(
+	                                                           pw_entry_objtype,
+	                                                           dest_e,
+	                                                           pw_entry_handle);
+	if (NULL == dest_extp) {
+		slapi_rwlock_unlock(src_extp->pw_entry_lock);
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "pw_copy_entry_ext: "
+		                "dest pw_entry_extension is not set\n");
+		return LDAP_NO_SUCH_ATTRIBUTE;
+	}
+
+	slapi_rwlock_wrlock(dest_extp->pw_entry_lock);
+	valuearray_add_valuearray(&dest_extp->pw_entry_values,
+	                          src_extp->pw_entry_values, 0);
+	slapi_rwlock_unlock(dest_extp->pw_entry_lock);
+	slapi_rwlock_unlock(src_extp->pw_entry_lock);
+	return LDAP_SUCCESS;
+}
+
+/* 
+ * The returned string is a copy.  
+ * Caller must free it.
+ */
+char *
+slapi_get_first_clear_text_pw(Slapi_Entry *entry)
+{
+	struct slapi_pw_entry_ext *extp = NULL;
+	Slapi_Value **pwvals = NULL;
+	const char *password_str = NULL;
+
+	if ((-1 == pw_entry_objtype) || (-1 == pw_entry_handle)) {
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "slapi_get_first_clear_text_pw: "
+		                "pw_entry_extension is not registered\n");
+		return NULL;
+	}
+
+	extp = (struct slapi_pw_entry_ext *)slapi_get_object_extension(
+	                                                           pw_entry_objtype,
+	                                                           entry,
+	                                                           pw_entry_handle);
+	if ((NULL == extp) || (NULL == extp->pw_entry_values)) {
+		slapi_log_error(SLAPI_LOG_TRACE, NULL,
+		                "slapi_get_first_clear_text_pw: "
+		                "pw_entry_extension is not set\n");
+		return NULL;
+	}
+
+	slapi_rwlock_rdlock(extp->pw_entry_lock);
+	pwvals = extp->pw_entry_values;
+	if (pwvals) {
+		Slapi_ValueSet vset;
+		Slapi_Value *value = NULL;
+		/* pwvals is passed in to vset; thus no need to free vset. */
+		valueset_set_valuearray_passin(&vset, pwvals);
+		slapi_valueset_first_value(&vset, &value);
+		password_str = slapi_value_get_string(value);
+	}
+	slapi_rwlock_unlock(extp->pw_entry_lock);
+	return slapi_ch_strdup(password_str); /* slapi_ch_strdup(NULL) is okay */
+}
